"use strict";
const fs = require("fs");
const path = require("path");
const { BacktraceService, Logger } = require("backtrace-service");
const log = Logger.getLogger();
let configDb;
if (process.env.NODE_ENV === "development" ||
    process.env.NODE_ENV === "standalone") {
    configDb = {
        client: "sqlite3",
        connection: {
            filename: `${process.cwd()}/database/sourcecode.db`,
        },
    };
}
else if (process.env.NODE_ENV === "test") {
    configDb = {
        client: "sqlite3",
        connection: {
            filename: `${process.cwd()}/database/test/${new Date().toISOString()}_sourcecode.test.db`,
        },
    };
}
else {
    const config = BacktraceService.readConfiguration("sourcecode", 6451, undefined, false);
    configDb = config.database;
}
const directory = path.dirname(configDb.connection.filename);
if (!fs.existsSync(directory)) {
    fs.mkdirSync(directory, { recursive: true }, (err) => {
        if (err) {
            log.error(`Error: Database directory ${directory} cannot be created.`);
            throw err;
        }
    });
}
const defaults = {
    migrations: {
        directory: `${process.cwd()}/migrations`,
    },
    useNullAsDefault: true,
};
module.exports = Object.assign(defaults, configDb);
//# sourceMappingURL=knexfile.js.map