"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../lib/errors");
const util_1 = require("../lib/util");
const perforceRulesController = (db, service, urlPrefix) => {
    service.app.get(`${urlPrefix}/:universeName/:projectName/perforce_rules`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const projectName = req.params.projectName;
        const universeName = req.params.universeName;
        let rules;
        rules = await db("perforce_rules")
            .select("*")
            .where("project_name", projectName)
            .where("universe_name", universeName);
        return res.send({ rules });
    }));
    service.app.put(`${urlPrefix}/:universeName/:projectName/perforce_rules/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { project_name, universe_name, position, field_type, field_name, regex, } = req.body;
        const updateProps = {
            project_name,
            universe_name,
            position,
            field_type,
            field_name,
            regex,
        };
        if (!Number.isInteger(position)) {
            return res.status(422).send(errors_1.getError(9, "Position not specified"));
        }
        const id = req.params.id;
        const rule = await db("perforce_rules").first("*").where("id", id);
        if (!rule) {
            return res
                .status(404)
                .send(errors_1.getError(4, `Rule with id ${id} not found`));
        }
        const origPos = rule.position;
        const newPos = position;
        await util_1.updatePosition({
            db,
            origPos,
            newPos,
            rowId: rule.id,
            tableName: "perforce_rules",
            extendQueryCb: (q) => q
                .where("universe_name", rule.universe_name)
                .where("project_name", rule.project_name),
            trxCb: (trx) => trx("perforce_rules")
                .first("*")
                .where("id", id)
                .update(updateProps),
        });
        const newRule = await db("perforce_rules").first("*").where("id", id);
        return res.send({ rule: newRule });
    }));
    service.app.post(`${urlPrefix}/:universeName/:projectName/perforce_rules`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { project_name, universe_name, position, matching_perforce_id, field_type, field_name, regex, action, } = req.body;
        const createProps = {
            project_name,
            universe_name,
            position,
            matching_perforce_id,
            field_type,
            field_name,
            regex,
            action,
        };
        const newRule = await util_1.createSpaceAtPosition({
            db,
            pos: 0,
            tableName: "perforce_rules",
            extendQueryCb: (q) => q
                .where("universe_name", universe_name)
                .where("project_name", project_name),
            trxCb: (trx) => trx("perforce_rules").insert(createProps),
        });
        return res.send({ rule: newRule });
    }));
    service.app.delete(`${urlPrefix}/:universeName/:projectName/perforce_rules/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const id = req.params.id;
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send(errors_1.getError(2, "Invalid rule ID"));
        }
        const rule = await db("perforce_rules")
            .first("position")
            .where("id", id);
        await db("perforce_rules").where("id", id).del();
        await db("perforce_rules")
            .where("position", ">", rule.position)
            .update({ position: db.raw(`\`position\` - 1`) });
        return res.status(200).send();
    }));
};
exports.default = perforceRulesController;
//# sourceMappingURL=perforce_rules.js.map