"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stats = void 0;
const backtrace_service_1 = require("backtrace-service");
const log = backtrace_service_1.Logger.getLogger();
class Stats {
    constructor({ name, prefix, init, res, }) {
        this._start = new Date();
        this._prefix = "stats";
        this._sent = false;
        this._data = {};
        let self = this;
        this._reportName = name;
        if (init) {
            Object.assign(this._data, init);
        }
        if (prefix)
            this._prefix = prefix;
        res === null || res === void 0 ? void 0 : res.on("finish", function () {
            self.send();
        });
    }
    inc(k) {
        if (this._data.hasOwnProperty(k) && !Number.isNaN(this._data[k])) {
            this._data[k] += 1;
        }
        else {
            this._data[k] = 1;
        }
    }
    dec(k) {
        if (this._data.hasOwnProperty(k) && !Number.isNaN(this._data[k])) {
            this._data[k] -= 1;
        }
        else {
            this._data[k] = -1;
        }
    }
    set(k, v) {
        if (typeof v === "function") {
            this._data[k] = v(this._data[k]);
        }
        else {
            this._data[k] = v;
        }
    }
    merge(v) {
        if (typeof v === "object") {
            Object.assign(this._data, v);
        }
    }
    get(k) {
        return this._data[k];
    }
    remove(k) {
        delete this._data[k];
    }
    setLatency(k, d = new Date(), d2 = this._start) {
        this._data[k] = Math.abs(d.valueOf() - d2.valueOf());
    }
    send() {
        if (this._sent === true) {
            log.warn("Attempting to send a single stats object multiple times.");
            return false;
        }
        this.setLatency("lifetime_latency");
        let data = {};
        Object.assign(data, this._data);
        if (this._prefix !== "") {
            for (let k in data) {
                data[`${this._prefix}_${k}`] = data[k];
                delete data[k];
            }
        }
        log.error(this._reportName, data);
        this._sent = true;
        return true;
    }
}
exports.Stats = Stats;
//# sourceMappingURL=git_stats.js.map