"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const util_1 = require("./util");
class ParallelQueue {
    constructor({ width, freq }) {
        this.queue = [];
        this.pending = [];
        this.intervalId = setInterval(() => {
            const numPending = this.pending.length;
            const slack = width - numPending;
            assert_1.default(slack >= 0);
            const toTake = Math.min(this.queue.length, slack);
            if (toTake > 0) {
                const tasks = this.queue.splice(0, toTake);
                this.pending = this.pending.concat(tasks);
                tasks.forEach((task) => {
                    const [taskFn, deferred] = task;
                    taskFn(deferred.resolve, deferred.reject);
                    deferred.promise.finally(() => {
                        this.removePendingTask(task);
                    });
                });
            }
        }, freq);
    }
    removePendingTask([task, _]) {
        this.pending = this.pending.filter(([pTask, _]) => pTask !== task);
    }
    push(tasks) {
        const deferredTasks = tasks.map((t) => [
            t,
            util_1.generateDeferredPromise(),
        ]);
        this.queue = this.queue.concat(deferredTasks);
        return deferredTasks.map(([_, deferredPromise]) => deferredPromise.promise);
    }
    destroy() {
        clearInterval(this.intervalId);
    }
}
exports.default = ParallelQueue;
//# sourceMappingURL=parallel_queue.js.map