"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.killSSHAgent = exports.addPrivateKeyToAgent = exports.startSSHAgent = exports.getAgent = exports.SSH_AGENT_ENV = void 0;
const child_process_1 = require("child_process");
exports.SSH_AGENT_ENV = { SSH_AUTH_SOCK: "", SSH_AGENT_PID: "" };
async function getAgent() {
    await ensureSSHAgent();
    return Object.assign({}, exports.SSH_AGENT_ENV);
}
exports.getAgent = getAgent;
function agentIsActive() {
    const pid = parseInt(exports.SSH_AGENT_ENV.SSH_AGENT_PID);
    if (pid <= 0 || isNaN(pid) || !Number.isInteger(pid) || pid === undefined) {
        return false;
    }
    try {
        process.kill(pid, 0);
        return true;
    }
    catch (e) {
        return false;
    }
}
exports.startSSHAgent = async () => {
    var _a, _b;
    const agent = child_process_1.spawnSync("ssh-agent");
    if (agent.error) {
        console.error(`Error starting ssh-agent: ${agent.error.message}`);
        return null;
    }
    const SSH_AUTH_SOCK = (_a = agent.stdout
        .toString()
        .match(/SSH_AUTH_SOCK=([^;]+)/)) === null || _a === void 0 ? void 0 : _a[1];
    const SSH_AGENT_PID = (_b = agent.stdout
        .toString()
        .match(/SSH_AGENT_PID=([^;]+)/)) === null || _b === void 0 ? void 0 : _b[1];
    if (!SSH_AUTH_SOCK || !SSH_AGENT_PID) {
        console.error("Error parsing ssh-agent output");
        return null;
    }
    let ssh_agent_env = {
        SSH_AUTH_SOCK,
        SSH_AGENT_PID,
    };
    exports.SSH_AGENT_ENV = ssh_agent_env;
    return ssh_agent_env;
};
async function ensureSSHAgent() {
    if (!agentIsActive()) {
        const agent = await exports.startSSHAgent();
        if (!agent) {
            console.error("Error starting ssh-agent");
            return false;
        }
    }
    return true;
}
async function addPrivateKeyToAgent(privateKey) {
    const agentEnv = await getAgent();
    const addKey = child_process_1.spawnSync("ssh-add", ["-"], {
        env: agentEnv,
        input: privateKey,
    });
    if (addKey.status !== 0) {
        console.error(`Error adding key to ssh-agent (exit code ${addKey.status})`);
        console.error(addKey.stderr.toString(), addKey.stdout.toString());
        return false;
    }
    return true;
}
exports.addPrivateKeyToAgent = addPrivateKeyToAgent;
function killSSHAgent() {
    if (exports.SSH_AGENT_ENV) {
        const killAgent = child_process_1.spawnSync("ssh-agent", ["-k"], {
            env: exports.SSH_AGENT_ENV,
        });
        if (killAgent.status !== 0) {
            console.error(`Error killing ssh-agent (exit code ${killAgent.status})`);
        }
    }
    else {
        console.log("No ssh-agent to kill");
    }
}
exports.killSSHAgent = killSSHAgent;
//# sourceMappingURL=ssh.js.map