"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decompress = exports.wrapSpawn = exports.getMimeType = exports.generateDeferredPromise = exports.parseForm = exports.untar = exports.tar = exports.highlightCodeCached = exports.withTimeout = exports.getStandaloneService = exports.isStandalone = exports.adminCheck = exports.bypassAccessCheck = exports.accessCheck = exports.asyncWrapper = exports.createSpaceAtPosition = exports.updatePosition = exports.getCoronerUrl = exports.getCoronerToken = exports.escapeRegExp = exports.daysAgoToSecondsAgo = exports.toFriendlyErrMsg = void 0;
const express_1 = __importDefault(require("express"));
const backtrace_service_1 = require("backtrace-service");
const errors_1 = require("../lib/errors");
const body_parser_1 = require("body-parser");
const http_1 = __importDefault(require("http"));
const cors_1 = __importDefault(require("cors"));
const helmet_1 = __importDefault(require("helmet"));
const nocache_1 = __importDefault(require("nocache"));
const compression_1 = __importDefault(require("compression"));
const fs_1 = __importDefault(require("fs"));
const cache_1 = __importDefault(require("../lib/cache"));
const highlight_js_1 = __importDefault(require("highlight.js"));
const child_process_1 = require("child_process");
const formidable_1 = require("formidable");
const path = __importStar(require("path"));
const { dirname, basename } = path;
const log = backtrace_service_1.Logger.getLogger();
const isStandalone = process.env.NODE_ENV === "development" ||
    process.env.NODE_ENV === "test" ||
    process.env.NODE_ENV === "standalone";
exports.isStandalone = isStandalone;
async function updatePosition({ db, origPos, newPos, rowId, tableName, positionCol = "position", extendQueryCb = (q) => q, trxCb = (_) => null, }) {
    const low = Math.min(origPos, newPos);
    const high = Math.max(origPos, newPos);
    const trx = await db.transaction();
    try {
        if (origPos !== newPos) {
            let q;
            if (origPos === low) {
                q = trx(tableName)
                    .where(positionCol, ">", low)
                    .where(positionCol, "<=", high)
                    .update({ position: trx.raw(`\`${positionCol}\` - 1`) });
            }
            else {
                q = trx(tableName)
                    .where(positionCol, ">=", low)
                    .where(positionCol, "<", high)
                    .update({ position: trx.raw(`\`${positionCol}\` + 1`) });
            }
            q = extendQueryCb(q);
            await q;
            await trx(tableName).where("id", rowId).update({ position: newPos });
        }
        const res = await trxCb(trx);
        trx.commit();
        return res;
    }
    catch (e) {
        trx.rollback();
        throw e;
    }
}
exports.updatePosition = updatePosition;
async function createSpaceAtPosition({ db, pos, tableName, positionCol = "position", extendQueryCb = (q) => q, trxCb = (_) => null, }) {
    const trx = await db.transaction();
    try {
        const blocker = await trx(tableName)
            .first("id")
            .where("position", "=", pos);
        if (typeof blocker === "object") {
            await extendQueryCb(trx(tableName)
                .where(positionCol, ">=", pos)
                .update({ position: trx.raw(`\`${positionCol}\` + 1`) }));
        }
        const res = await trxCb(trx);
        trx.commit();
        return res;
    }
    catch (e) {
        trx.rollback();
        throw e;
    }
}
exports.createSpaceAtPosition = createSpaceAtPosition;
function asyncWrapper(fn) {
    return (req, res, next) => {
        Promise.resolve(fn(req, res, next)).catch((e) => {
            if (isStandalone) {
                console.log(e);
            }
            next(e);
        });
    };
}
exports.asyncWrapper = asyncWrapper;
function accessCheck(service) {
    return function accessCheck2(req, res, next) {
        if (isStandalone) {
            return next();
        }
        const projectName = req.params.projectName;
        const universeName = req.params.universeName;
        if (!projectName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, "missing param project name"));
        }
        if (!universeName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, "missing param universe name"));
        }
        service.identityManager.checkProjectAccess(req, res, [
            projectName,
        ]);
        next();
    };
}
exports.accessCheck = accessCheck;
function bypassAccessCheck(service, allowProduction = false) {
    return (_req, res, next) => {
        if (isStandalone) {
            return next();
        }
        if (!allowProduction && !isStandalone) {
            log.error("Calling bypassAccessCheck in production." +
                "Are you sure you want to do this? Pass true as a second parameter to override.");
            res.status(403).send();
        }
        service.identityManager.bypassProjectAccessCheck(res);
        next();
    };
}
exports.bypassAccessCheck = bypassAccessCheck;
function adminCheck(service) {
    return function adminCheck2(req, res, next) {
        var _a, _b, _c, _d, _e, _f, _g;
        if (isStandalone) {
            return next();
        }
        const projectName = req.params.projectName;
        const universeName = req.params.universeName;
        if (!projectName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, "missing param project name"));
        }
        if (!universeName) {
            service.identityManager.bypassProjectAccessCheck(res);
            return res.json(errors_1.getError(2, "missing param universe name"));
        }
        const authRequest = req;
        const proj = (_d = (_c = (_b = (_a = authRequest === null || authRequest === void 0 ? void 0 : authRequest.coronerAuth) === null || _a === void 0 ? void 0 : _a.cfg) === null || _b === void 0 ? void 0 : _b.universes[universeName]) === null || _c === void 0 ? void 0 : _c.projects_ext) === null || _d === void 0 ? void 0 : _d.find((project) => project.name === projectName);
        const projRole = proj === null || proj === void 0 ? void 0 : proj.role;
        const userRole = (_g = (_f = (_e = authRequest === null || authRequest === void 0 ? void 0 : authRequest.coronerAuth) === null || _e === void 0 ? void 0 : _e.cfg) === null || _f === void 0 ? void 0 : _f.user) === null || _g === void 0 ? void 0 : _g.role;
        if (userRole === "admin" || projRole === "admin") {
            next();
        }
        else {
            return res.status(403).send();
        }
    };
}
exports.adminCheck = adminCheck;
function daysAgoToSecondsAgo(days) {
    return Math.floor(Date.now() / 1000) - days * 24 * 3600;
}
exports.daysAgoToSecondsAgo = daysAgoToSecondsAgo;
function escapeRegExp(str) {
    return str.replace(/[.*+\-?^${}()|[\]\\]/g, "\\$&");
}
exports.escapeRegExp = escapeRegExp;
function getCoronerToken(service, req) {
    var _a, _b;
    return (((_b = (_a = req === null || req === void 0 ? void 0 : req.coronerAuth) === null || _a === void 0 ? void 0 : _a.cfg) === null || _b === void 0 ? void 0 : _b.token) ||
        process.env.CORONER_TOKEN ||
        service.configuration.coronerToken);
}
exports.getCoronerToken = getCoronerToken;
async function getCoronerUrl(universeName, db, service, req) {
    var _a, _b;
    return (((_a = req === null || req === void 0 ? void 0 : req.coronerAuth) === null || _a === void 0 ? void 0 : _a.url) ||
        process.env.CORONER_LOCATION ||
        service.configuration.coronerLocation || ((_b = (await db("repositories")
        .first("coroner_location")
        .where("universe_name", universeName))) === null || _b === void 0 ? void 0 : _b.coroner_location) ||
        `https://${universeName}.sp.backtrace.io`);
}
exports.getCoronerUrl = getCoronerUrl;
function getStandaloneService({ name, port, config = {}, }) {
    const app = express_1.default();
    app.use(cors_1.default());
    app.use(nocache_1.default());
    app.use(helmet_1.default({
        hsts: false,
    }));
    app.use(body_parser_1.json());
    app.use(body_parser_1.urlencoded({ extended: true }));
    app.use(compression_1.default());
    let confFromFile;
    try {
        confFromFile = JSON.parse(fs_1.default.readFileSync(process.cwd() + "/sourcecode.conf").toString());
    }
    catch (e) {
        confFromFile = {};
    }
    return {
        app,
        metrics: new backtrace_service_1.MetricsStorage("", ""),
        start: () => {
            const server = http_1.default.createServer(app);
            server.listen(port, () => {
                log.info("Source code service listening on port " + port);
            });
        },
        configuration: Object.assign(config, confFromFile),
        apiPrefix: `api/${name}`,
        identityManager: {
            authenticateRequest: () => {
                return (_req, _res, next) => {
                    next();
                };
            },
        },
    };
}
exports.getStandaloneService = getStandaloneService;
async function withTimeout(pr, timeout) {
    return new Promise((resolv, rej) => {
        pr.then((resp) => resolv(resp)).catch((e) => rej(e));
        setTimeout(() => resolv("timeout"), timeout);
    });
}
exports.withTimeout = withTimeout;
function hashFnv32a(str, asString, seed) {
    var i, l, hval = seed === undefined ? 0x811c9dc5 : seed;
    for (i = 0, l = str.length; i < l; i++) {
        hval ^= str.charCodeAt(i);
        hval +=
            (hval << 1) + (hval << 4) + (hval << 7) + (hval << 8) + (hval << 24);
    }
    if (asString) {
        return ("0000000" + (hval >>> 0).toString(16)).substr(-8);
    }
    return hval >>> 0;
}
async function highlightCodeCached(universeName, projectName, sourceCodeStr, stats) {
    var _a;
    let hash = hashFnv32a(sourceCodeStr, true);
    let highlightedStr = (_a = (await cache_1.default.get({
        project_name: projectName,
        universe_name: universeName,
        ckey: hash,
    }))) === null || _a === void 0 ? void 0 : _a.cvalue;
    if (!highlightedStr) {
        stats === null || stats === void 0 ? void 0 : stats.set("highlight_cache_miss", 1);
        const startTs = new Date();
        highlightedStr = highlight_js_1.default.highlightAuto(sourceCodeStr).value;
        stats === null || stats === void 0 ? void 0 : stats.setLatency("highlight_latency", startTs, new Date());
        cache_1.default.set({
            project_name: projectName,
            universe_name: universeName,
            ckey: hash,
            cvalue: highlightedStr,
            ctype: "highlighted_sourcecode",
        });
    }
    stats === null || stats === void 0 ? void 0 : stats.set("highlight_cache_miss", 0);
    return highlightedStr;
}
exports.highlightCodeCached = highlightCodeCached;
function tar(inputPath, outputPath, exclude) {
    return wrapSpawn("tar", [
        "-c",
        "-a",
        ...(exclude ? ["--exclude", exclude] : []),
        "-f",
        outputPath,
        "-C",
        dirname(inputPath),
        basename(inputPath),
    ]);
}
exports.tar = tar;
function untar(filePath, extractPath, timeout = 2000, strip) {
    const opts = {};
    if (timeout)
        opts.timeout = timeout;
    return wrapSpawn("tar", [
        "xvf",
        filePath,
        "--directory",
        extractPath,
        ...(strip ? ["--strip-components", strip.toString()] : []),
    ], opts);
}
exports.untar = untar;
function unzip(filePath, extractPath, timeout = 2000) {
    return wrapSpawn("unzip", [filePath, "-d", extractPath], { timeout });
}
function wrapSpawn(cmd, args = [], opts = {
    env: {},
    timeout: 2000,
}, moreOpts = {}) {
    const { stdout, stderr, error } = child_process_1.spawnSync(cmd, args, opts);
    if (error) {
        throw error;
    }
    if (moreOpts.ignoreStderr !== true) {
        if (stderr.length > 0) {
            const msg = stderr.toString();
            throw new Error(msg);
        }
    }
    return stdout.toString().trim();
}
exports.wrapSpawn = wrapSpawn;
function getMimeType(filePath) {
    return wrapSpawn("file", ["-bi", filePath]);
}
exports.getMimeType = getMimeType;
async function parseForm(req) {
    const form = new formidable_1.IncomingForm();
    return new Promise((resolve, reject) => {
        form.parse(req, (err, fields, files) => {
            if (err) {
                return reject(err);
            }
            const ret = { files, fields };
            resolve(ret);
        });
    });
}
exports.parseForm = parseForm;
function generateDeferredPromise() {
    return (() => {
        let resolve = () => { };
        let reject = () => { };
        let p = new Promise((res, rej) => {
            resolve = res;
            reject = rej;
        });
        return {
            promise: p,
            reject,
            resolve,
        };
    })();
}
exports.generateDeferredPromise = generateDeferredPromise;
function decompress(filePath, extractPath, timeout = 2000, strip) {
    const fileType = getMimeType(filePath);
    if (new RegExp("application/zip").test(fileType)) {
        return unzip(filePath, extractPath, timeout);
    }
    else {
        return untar(filePath, extractPath, timeout, strip);
    }
}
exports.decompress = decompress;
function toFriendlyErrMsg(msg) {
    return /too many redirects or authentication replays/.test(msg)
        ? "Username / password invalid"
        : msg;
}
exports.toFriendlyErrMsg = toFriendlyErrMsg;
//# sourceMappingURL=util.js.map