"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const service_1 = __importDefault(require("../src/lib/service"));
const commit_rules_1 = __importDefault(require("./controller/commit_rules"));
const repo_rules_1 = __importDefault(require("./controller/repo_rules"));
const perforce_rules_1 = __importDefault(require("./controller/perforce_rules"));
const repositories_1 = __importDefault(require("./controller/repositories"));
const source_code_1 = __importDefault(require("./controller/source_code"));
const blame_1 = __importDefault(require("./controller/blame"));
const perforce_1 = __importDefault(require("./controller/perforce"));
const archives_1 = __importDefault(require("./controller/archives"));
const git_routines_1 = require("./lib/git_routines");
const util_1 = require("./lib/util");
const db_1 = __importStar(require("./lib/db"));
const cache_1 = __importDefault(require("./lib/cache"));
const cachedb = cache_1.default.db;
const parallel_queue_1 = __importDefault(require("../src/lib/parallel_queue"));
const { log, service, name } = service_1.default();
console.log(`[NODE_ENV] ${process.env.NODE_ENV}`);
console.log(`[REPOSITORIES_PATH] ${git_routines_1.reposPath()}`);
console.log(`[KNEXFILE] database ${db_1.knexfile.connection.filename}`);
const urlPrefix = ((_a = service.configuration.descriptor) === null || _a === void 0 ? void 0 : _a.resource) || `/api/${name}`;
repositories_1.default(db_1.default, service, urlPrefix);
repo_rules_1.default(db_1.default, service, urlPrefix);
perforce_rules_1.default(db_1.default, service, urlPrefix);
commit_rules_1.default(db_1.default, service, urlPrefix);
source_code_1.default(db_1.default, service, urlPrefix);
blame_1.default(db_1.default, service, urlPrefix);
perforce_1.default(db_1.default, service, urlPrefix);
archives_1.default(db_1.default, service, urlPrefix);
service.app.get(`${urlPrefix}/version`, service.identityManager.authenticateRequest(), util_1.bypassAccessCheck(service, true), (_req, res) => {
    res.json({ version: process.env.npm_package_version });
});
const fetchQueue = new parallel_queue_1.default({ width: 4, freq: 10000 });
const refetchInterval = 10 * 60;
const currentlyFetching = new Set();
const refetchTimer = async () => {
    setInterval(async () => {
        try {
            const repositories = await db_1.default("repositories")
                .select("*")
                .whereNot({ url: "local_archive" })
                .whereNotIn("id", Array.from(currentlyFetching))
                .andWhere({ clone_status: "complete" });
            for (const dbRepo of repositories) {
                currentlyFetching.add(dbRepo.id);
                fetchQueue.push([
                    async (resolve, reject) => {
                        try {
                            resolve(await git_routines_1.gitFetchOrCloneSpawn(dbRepo.id.toString()));
                        }
                        catch (e) {
                            log.error(e, { repoId: dbRepo.id });
                            resolve();
                        }
                        finally {
                            currentlyFetching.delete(dbRepo.id);
                        }
                    },
                ]);
            }
        }
        catch (e) {
            e.message = "refetchTimer: " + e.message;
            log.error(e);
        }
    }, refetchInterval * 1000);
};
const garbageCollectInterval = 60 * 60;
const currentlyGarbageCollecting = new Set();
const garbageCollectTimer = async () => {
    setInterval(async () => {
        if (currentlyGarbageCollecting.size > 0) {
            console.log(`[garbageCollectTimer] Skipping this generation. Garbage collect still in progress from last generation. The following repositories are still garbage collecting: ${Array.from(currentlyGarbageCollecting).join(" ")}`);
            return;
        }
        try {
            const repositories = await db_1.default("repositories")
                .select("*")
                .whereNot({ url: "local_archive" })
                .whereNotIn("id", Array.from(currentlyGarbageCollecting))
                .andWhere({ clone_status: "complete" });
            for (const dbRepo of repositories) {
                currentlyGarbageCollecting.add(dbRepo.id);
                const dbRepoText = `${dbRepo.universe_name} ${dbRepo.project_name} ${dbRepo.name} ${dbRepo.id}`;
                try {
                    console.log(`[garbageCollectTimer] start ${dbRepoText}`);
                    await git_routines_1.gitGarbageCollect(dbRepo.id.toString());
                    console.log(`[garbageCollectTimer] done ${dbRepoText}`);
                }
                catch (e) {
                    log.error(e, { repoId: dbRepo.id });
                    console.log(`[garbageCollectTimer] ERROR ${dbRepoText}`);
                }
                finally {
                    currentlyGarbageCollecting.delete(dbRepo.id);
                }
            }
        }
        catch (e) {
            e.message = "[garbageCollectTimer]: " + e.message;
            log.error(e);
        }
    }, garbageCollectInterval * 1000);
};
(async () => {
    try {
        await db_1.default.migrate.latest();
        await cachedb.migrate.latest();
    }
    catch (e) {
        const filename = db_1.knexfile.connection.filename;
        log.error(e);
        log.error(`Unable to write to database file. Please make sure this process has write permission to this file and start the service again: ${filename}`);
        process.exit(1);
    }
    let { path: gPath, version: gVersion } = await git_routines_1.gitVersion();
    log.info(`[git] Using git ${gVersion.replace("\n", "")} at ${gPath}`);
    service.start();
    if (service.configuration.disableRefetch === true) {
        log.warn("config.disableRefetch: Disabling automatic refetch of repositories");
    }
    else {
        log.info("Enabling automatic refetch of repositories");
        refetchTimer();
        garbageCollectTimer();
    }
})();
exports.default = service;
//# sourceMappingURL=main.js.map