const fs = require("fs");
const path = require("path");
const { BacktraceService, Logger } = require("backtrace-service");

const log = Logger.getLogger();

let configDb;
if (process.env.NODE_ENV === "development") {
  configDb = {
    client: "sqlite3",
    connection: {
      filename: `${process.cwd()}/database/sourcecode_cachedb.dev.db`,
    },
  };
} else if (process.env.NODE_ENV === "test") {
  configDb = {
    client: "sqlite3",
    connection: {
      filename: `${process.cwd()}/database/test/${(new Date()).toISOString()}_sourcecode_cachedb.test.db`,
    },
  };
} else {
  configDb = {
    client: "sqlite3",
    connection: {
      filename: `${process.cwd()}/database/sourcecode_cachedb.db`,
    },
  };
}

// Check db directory if it does not already exist
const directory = path.dirname(configDb.connection.filename);
if (!fs.existsSync(directory)) {
  fs.mkdirSync(directory, { recursive: true }, (err) => {
    if (err) {
      log.error(`Error: Database directory ${directory} cannot be created.`);
      throw err;
    }
  });
}

const defaults = {
  migrations: {
    directory: `${process.cwd()}/migrations_cachedb`,
  },
  useNullAsDefault: true,
};

module.exports = Object.assign(defaults, configDb);
