import { BacktraceRole } from './backtraceRole';
export interface IConfig {
    version: string;
}
export interface IEndpointInformation {
    port: number;
    protocol: string;
    id: number;
}
export interface IEndpoints {
    [index: string]: IEndpointInformation[];
}
export interface IUser {
    role: BacktraceRole;
    username: string;
    email: string;
    uid: number;
    superuser: number;
    active: number;
    method: string;
    universe: number;
    deleted: number;
    password?: string | null;
    metadata?: string | null;
    __create?: number;
    __modify?: number;
    __namespace?: object;
    __state?: object;
}
export interface IEphemeral {
    last_project_pid: number;
}
export interface IUniverse {
    name: string;
    id: number;
    deleted: number;
}
export interface IWatcher {
    name: string;
    integration_type: string;
}
export interface IProjectsExt {
    role: BacktraceRole;
    name: string;
    pid: number;
    universe: number;
    owner: number;
    deleted: number;
    watchers: IWatcher[];
    frontend_settings?: string;
    corrupted?: boolean;
    _tx: number;
}
export interface IUniverseProject {
    projects: string[];
    projects_ext: IProjectsExt[];
}
export interface IUniverses {
    [index: string]: IUniverseProject;
}
export interface IIntercom {
    user_hash: string;
    user_id: string;
    company_id: string;
}
export interface IServiceConfiguration {
    name: string;
    endpoint: string;
}
export interface IServerConfiguration {
    /**
     * Server token. Token might be unvailable when a service
     * tries to login to coronerd instance available on the same host
     */
    token: string | undefined;
    config: IConfig;
    endpoints: IEndpoints;
    role: BacktraceRole;
    username: string;
    email: string;
    uid: number;
    user: IUser;
    services: IServiceConfiguration[];
    ephemeral: IEphemeral;
    universe: IUniverse;
    universes: IUniverses;
    intercom?: IIntercom;
    host: string;
    users: IUser[];
    tenant_separator: string;
}
