"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
var backtrace_node_1 = require("backtrace-node");
var util_1 = __importDefault(require("util"));
var winston_1 = __importStar(require("winston"));
var Logger = /** @class */ (function () {
    function Logger(optionsOrLevel, options) {
        var _this = this;
        var _a;
        // transportLevel definition that Logger class respect:
        // #define SD_EMERG   "<0>"  /* system is unusable */
        // #define SD_ALERT   "<1>"  /* action must be taken immediately */
        // #define SD_CRIT    "<2>"  /* critical conditions */
        // #define SD_ERR     "<3>"  /* error conditions */
        // #define SD_WARNING "<4>"  /* warning conditions */
        // #define SD_NOTICE  "<5>"  /* normal but significant condition */
        // #define SD_INFO    "<6>"  /* informational */
        // #define SD_DEBUG   "<7>"  /* debug-level messages */
        this.transportLevel = {
            error: 3,
            warn: 4,
            info: 6,
            trace: 7,
            verbose: 7,
            debug: 7,
            silly: 7,
        };
        if (typeof optionsOrLevel === 'string') {
            this.options = Object.assign(this.defaultOptions(), options);
            this.options.logLevel = optionsOrLevel;
        }
        else {
            this.options = Object.assign(this.defaultOptions(), optionsOrLevel);
        }
        this.winstonLogger = winston_1.default.createLogger({
            level: this.options.logLevel,
            format: winston_1.format.combine(
            // Save the original level to use it later after the level has been colorized
            {
                transform: function (info) {
                    info.originalLevel = info.level;
                    return info;
                },
            }, winston_1.format.colorize(), winston_1.format.printf(function (_a) {
                var level = _a.level, message = _a.message, originalLevel = _a.originalLevel;
                var date = new Date().toLocaleString();
                var result = "".concat(level, ": ") +
                    "<".concat(_this.getTransportLevel(originalLevel), ">") +
                    "".concat(originalLevel, ":[").concat(date, "] ").concat(message);
                if (!_this.options.newline) {
                    return result.replace(/\n+/g, ' ').replace(/\r+/g, ' ');
                }
                return result;
            })),
            transports: (_a = this.options.transports) !== null && _a !== void 0 ? _a : new winston_1.transports.Console({
                silent: process.env.NODE_ENV === 'test',
            }),
            levels: {
                error: 0,
                warn: 1,
                info: 2,
                debug: 3,
                verbose: 4,
                trace: 4,
                silly: 4,
            },
        });
        Logger._logger = this;
    }
    Logger.getLogger = function () {
        if (!Logger._logger) {
            this._logger = new Logger();
            this._logger.warn("Creating a new default logger instance with default settings.");
        }
        return this._logger;
    };
    /**
     * overrides the default logger instance or set the existing one.
     * @param logger the logger instance.
     */
    Logger.setLogger = function (logger) {
        this._logger = logger;
    };
    Logger.initializeBacktrace = function (opts) {
        if (!opts || !opts.endpoint) {
            return;
        }
        var logger = this.getLogger();
        logger.debug('Initializing backtrace-node integration');
        var client = (0, backtrace_node_1.initialize)(opts);
        if (!client) {
            logger.debug('Cannot initialize Backtrace integration. Please verify Backtrace url');
        }
    };
    Logger.prototype.trace = function (errorOrData) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        this.log.apply(this, __spreadArray(['verbose', errorOrData], data, false));
    };
    Logger.prototype.info = function (errorOrData) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        this.log.apply(this, __spreadArray(['info', errorOrData], data, false));
    };
    Logger.prototype.warn = function (errorOrData) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        this.log.apply(this, __spreadArray(['warn', errorOrData], data, false));
    };
    Logger.prototype.debug = function (errorOrData) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        this.log.apply(this, __spreadArray(['debug', errorOrData], data, false));
    };
    Logger.prototype.error = function (errorOrData) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        var exception = errorOrData instanceof Error ? errorOrData : new Error(errorOrData);
        var attributes;
        if (data &&
            data.length > 0 &&
            data[0] instanceof Object &&
            !Array.isArray(data[0])) {
            attributes = data[0];
            data.shift();
        }
        this.log.apply(this, __spreadArray(['error', errorOrData], data, false));
        var client = (0, backtrace_node_1.getBacktraceClient)();
        if (client) {
            client.reportSync(exception, attributes);
        }
    };
    Logger.prototype.log = function (level, errorOrData) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        if (errorOrData instanceof Error) {
            if (args && args.length) {
                this.winstonLogger.log({
                    level: level,
                    message: util_1.default.format.apply(util_1.default, __spreadArray(__spreadArray([args.shift()], args, false), [this.options.stackTrace
                            ? errorOrData
                            : "Error: ".concat(errorOrData.message)], false)),
                });
            }
            else {
                this.winstonLogger.log({
                    level: level,
                    message: util_1.default.format(this.options.stackTrace ? errorOrData : errorOrData.message),
                });
            }
        }
        else {
            this.winstonLogger.log({
                level: level,
                message: util_1.default.format.apply(util_1.default, __spreadArray([errorOrData], args, false)),
            });
        }
    };
    Logger.prototype.getTransportLevel = function (logLevel) {
        return this.transportLevel[logLevel] || this.transportLevel['debug'];
    };
    Logger.prototype.defaultOptions = function () {
        return {
            logLevel: 'debug',
            newline: false,
            stackTrace: false,
        };
    };
    return Logger;
}());
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map