"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHelper = void 0;
var axios_1 = __importDefault(require("axios"));
var ErrorHelper = /** @class */ (function () {
    function ErrorHelper() {
    }
    /**
     * Generate an error message based on the error type. If the error
     * is not an error, then will use the default error message.
     * @param error Error object
     * @param defaultMessage default error message if the error object is not an error
     * @returns error message
     */
    ErrorHelper.getErrorMessage = function (error, defaultMessage) {
        if (defaultMessage === void 0) { defaultMessage = "Unknown error. Reason: ".concat(error); }
        return error instanceof Error ? error.message : defaultMessage;
    };
    /**
     * Retrieves status code based on the error object and possible axios response.
     * @param error Error object
     * @param defaultStatusCode default status code
     * @returns status code
     */
    ErrorHelper.getErrorStatusCode = function (error, defaultStatusCode) {
        var _a, _b;
        if (defaultStatusCode === void 0) { defaultStatusCode = 500; }
        return axios_1.default.isAxiosError(error)
            ? (_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.status) !== null && _b !== void 0 ? _b : defaultStatusCode
            : defaultStatusCode;
    };
    return ErrorHelper;
}());
exports.ErrorHelper = ErrorHelper;
//# sourceMappingURL=errorHelper.js.map