import fetch from "node-fetch";
import { repoPath } from "../src/lib/git_routines";
import fs from "fs";

describe("Repositories end-to-end tests", () => {
  beforeAll(async () => {
    try {
      await fetch("http://localhost:6451/api/sourcecode/version", {
        method: "GET",
      });
    } catch (e) {
      console.log(
        "Please run `npm run test-server`. This test requires a running server. Liveness check failed."
      );
      process.exit(1);
    }
  });
  describe("POST /repositories and GET /repositories and GET /repositories/:id/fetch", () => {
    let clonedId = null;
    it("should clone an open/unauthenticated repository and verify that it cloned correctly", async () => {
      const body = {
        id: null,
        name: "open repo",
        project_name: "coronerd",
        universe_name: "backtrace",
        url: "https://github.com/rashkov-at-sauce/open_repo.git",
        default_commit: "main",
        auth_type: "none",
        user_name: null,
        password: null,
        ssh_public_key: null,
        ssh_private_key: null,
        position: null,
        autoclone_submodules: 0,
      };
      const res = await fetch(
        "http://localhost:6451/api/sourcecode/backtrace/coronerd/repositories",
        {
          body: JSON.stringify(body),
          headers: {
            "Content-Type": "application/json",
          },
          method: "POST",
        }
      );
      // response is in the 200s
      expect(res.status).toBeGreaterThanOrEqual(200);
      const resJson = await res.json();

      // poll for the repository to be cloned
      let startTime = null;
      const poll = async (id: number | string, timeoutMs: number) => {
        if (startTime === null) {
          startTime = Date.now();
        } else {
          if (Date.now() - startTime > timeoutMs) {
            startTime = null;
            throw new Error("Timeout exceeded");
          }
        }
        const res = await fetch(
          `http://localhost:6451/api/sourcecode/backtrace/coronerd/repositories`,
          {
            method: "GET",
          }
        );
        const resJson = await res.json();
        const repository = resJson.repositories.find((r: any) => r.id === id);
        if (repository.clone_status === "complete") {
          return repository;
        } else {
          // insert a wait here to avoid overloading the server
          await new Promise((resolve) => setTimeout(resolve, 1000));
          return await poll(id, timeoutMs);
        }
      };
      // wait up to 10 seconds to clone
      const clonedRepo = await poll(resJson.repository.id, 10000);
      expect(clonedRepo.clone_status).toBe("complete");

      clonedId = clonedRepo.id;
    });
    it("should fetch the repository", async () => {
      if (clonedId === null) {
        throw new Error("clonedId is null");
      }
      const res = await fetch(
        `http://localhost:6451/api/sourcecode/backtrace/coronerd/repositories/${clonedId}/fetch`,
        {
          method: "GET",
        }
      );
      // response is in the 200s
      expect(res.status).toBeGreaterThanOrEqual(200);
      const resJson = await res.json();
      expect(resJson).toEqual({});
    });
  });
});
