import {
  gitInit,
  appendText,
  writeText,
  gitAdd,
  gitCommit,
} from "../test_utils";
import { resolve } from "path";
import { cloneRepo } from "../../src/lib/git_routines";
import { tar } from "../../src/lib/util";
import * as fs from "fs";
import {
  createArchiveDb,
  createArchiveRepo,
  addTarToArchive,
} from "../../src/lib/archive";

type file = {
  filepath: string;
  txt: string;
  append?: boolean;
  untracked?: boolean;
};
type commit = {
  msg: string;
  files: file[];
};
type script = commit[];

export function createRepo(script: script) {
  const repoPath = gitInit();
  for (const commit of script) {
    for (const { filepath, txt, append, untracked } of commit.files) {
      if (append) {
        appendText(resolve(repoPath, filepath), txt);
      } else {
        writeText(resolve(repoPath, filepath), txt);
      }
      if (!untracked) gitAdd(repoPath, filepath);
    }
    gitCommit(repoPath, commit.msg);
  }
  return repoPath;
}

export async function createAndCloneRepo({
  db,
  universeName,
  projectName,
  script,
}: {
  db: any;
  universeName: string;
  projectName: string;
  script: script;
}) {
  const repoPath = createRepo(script);
  const id = await cloneRepo(db, {
    name: "testrepo",
    url: repoPath,
    default_commit: "master",
    auth_type: "none",
    project_name: projectName,
    universe_name: universeName,
    clone_status: undefined,
    user_name: undefined,
    password: undefined,
    ssh_public_key: undefined,
    ssh_private_key: undefined,
  });
  return { id, repoPath };
}

export function createArchiveTar(name: string, script: script) {
  const repoPath = createRepo(script);
  const archiveFilesPath = resolve(
    process.cwd(),
    "test",
    "corpusTmp",
    "archive_files"
  );
  fs.mkdirSync(archiveFilesPath, { recursive: true });
  const tarName = name + ".tar";
  const tarPath = resolve(archiveFilesPath, tarName);
  tar(repoPath, tarPath, ".git");
  return tarPath;
}

export async function createAndCloneArchive({
  universeName,
  projectName,
  script,
  name,
}: {
  universeName: string;
  projectName: string;
  script: script;
  name: string;
}) {
  const data = {
    universe_name: universeName,
    project_name: projectName,
    name: "testArc1",
    default_commit: "master",
  };
  const tarPath = createArchiveTar(name, script);
  const id = await createArchiveDb(data);
  await createArchiveRepo(id);
  const out = await addTarToArchive(tarPath, "foo", id);
  return { commit: out, archivePath: tarPath };
}
